@@ -21,10 +21,14 @@
 #include <spice.h>
 #include <glib.h>
 
+SPICE_BEGIN_DECLS
+
 GMainContext *basic_event_loop_get_context(void);
 SpiceCoreInterface *basic_event_loop_init(void);
 void basic_event_loop_destroy(void);
 void basic_event_loop_mainloop(void);
 void basic_event_loop_quit(void);
 
+SPICE_END_DECLS
+
 #endif // __BASIC_EVENT_LOOP_H__
