@@ -9,7 +9,7 @@ from python_modules import ptypes
 from python_modules import codegen
 from python_modules import demarshal
 from python_modules import marshal
-import six
+
 
 def write_channel_enums(writer, channel, client, describe):
     messages = list(filter(lambda m : m.channel == channel, \
@@ -108,25 +108,20 @@ def write_enums(writer, describe=False):
 def write_content(dest_file, content, keep_identical_file):
     if keep_identical_file:
         try:
-            f = open(dest_file, 'rb')
-            old_content = f.read()
-            f.close()
+            with open(dest_file, 'rb') as f:
+                old_content = f.read()
 
             if content == old_content:
-                six.print_("No changes to %s" % dest_file)
+                print("No changes to %s" % dest_file)
                 return
 
         except IOError:
             pass
 
-    f = open(dest_file, 'wb')
-    if six.PY3:
+    with open(dest_file, 'wb') as f:
         f.write(bytes(content, 'UTF-8'))
-    else:
-        f.write(content)
-    f.close()
 
-    six.print_("Wrote %s" % dest_file)
+    print("Wrote %s" % dest_file)
 
 
 parser = OptionParser(usage="usage: %prog [options] <protocol_file> <destination file>")
@@ -256,7 +251,7 @@ elif options.license == "BSD":
 
 """
 else:
-    print >> sys.stderr, "Invalid license specified: %s" % options.license
+    print("Invalid license specified: %s" % options.license, file=sys.stderr)
     sys.exit(1)
 
 all_structures = {}
@@ -280,8 +275,8 @@ current:
             all_structures[c_type] = value
             t.generate_c_declaration(writer_top)
     except:
-        print >> sys.stderr, 'type %s' % t
-        print >> sys.stderr, writer.getvalue()
+        print('type %s' % t, file=sys.stderr)
+        print(writer.getvalue(), file=sys.stderr)
         traceback.print_exc(sys.stderr)
         sys.exit(1)
 
@@ -330,7 +325,7 @@ if options.generate_enums or options.generate_dissector:
 
 if options.generate_demarshallers:
     if not options.server and not options.client:
-        print >> sys.stderr, "Must specify client and/or server"
+        print("Must specify client and/or server", file=sys.stderr)
         sys.exit(1)
     demarshal.write_includes(writer)
 
@@ -344,7 +339,7 @@ if options.generate_marshallers or (options.struct_marshallers and len(options.s
 
 if options.generate_marshallers:
     if not options.server and not options.client:
-        print >> sys.stderr, "Must specify client and/or server"
+        print("Must specify client and/or server", file=sys.stderr)
         sys.exit(1)
     if options.server:
         marshal.write_protocol_marshaller(writer, proto, False, options.private_marshallers)
